#!/usr/bin/env python3

import os
import json
import binascii
import argparse
import boto3
import time
import docker
import base64

from botocore.exceptions import ClientError

RETRY_EXCEPTIONS = ('ProvisionedThroughputExceededException',
                    'ThrottlingException')

def expand_file(file_in, file_out):
    with open(file_in, 'r') as in_file:
        in_file_content = os.path.expandvars(in_file.read())
        with open(file_out, 'w') as out_file:
            out_file.write(in_file_content)

    with open(state_file, 'r') as state:
        state_content = os.path.expandvars(state.read())
        with open(state_file + '.tf', 'w') as state_tf:
            state_tf.write(state_content)


def expand_state_files(*state_files):
    for state_file in state_files:
        try:
            with open(state_file, 'r') as state:
                state_content = os.path.expandvars(state.read())
                with open(state_file + '.tf', 'w') as state_tf:
                    state_tf.write(state_content)
        except:
            pass


def cleanup_state_files(*state_files):
    for state_file in state_files:
        try:
            os.remove(state_file)
        except OSError:
            pass


def service_exists(cluster_name, service_name):
    client = boto3.client('ecs')
    response = client.describe_services(
        cluster=cluster_name,
        services=[
            service_name,
        ]
    )
    print(len(response["services"]) == 1 and response["services"][0]["status"] == "ACTIVE")


def latest_task_definition(family_prefix):
    retries = 0
    while True:
        try:
            client = boto3.client('ecs')
            response = client.list_task_definitions(
                familyPrefix=family_prefix,
                status='ACTIVE',
                sort='DESC'
            )
            print(response["taskDefinitionArns"][0].split(':')[-1] if len(response["taskDefinitionArns"]) > 0 else 0)
            return
        except ClientError as err:
            if err.response['Error']['Code'] not in RETRY_EXCEPTIONS:
                raise
            time.sleep(2 ** retries)
            retries += 1
            if retries > 5:
                raise

def latest_task_definition_arn(family_prefix):
    retries = 0
    while True:
        try:
            client = boto3.client('ecs')
            response = client.list_task_definitions(
                familyPrefix=family_prefix,
                status='ACTIVE',
                sort='DESC'
            )
            print(response["taskDefinitionArns"][0] if len(response["taskDefinitionArns"]) > 0 else "")
            return
        except ClientError as err:
            if err.response['Error']['Code'] not in RETRY_EXCEPTIONS:
                raise
            time.sleep(2 ** retries)
            retries += 1
            if retries > 5:
                raise


def task_definition_updated(new_revision, old_revision):
    print(str(new_revision) != str(old_revision))


def repo_url(repo_name):
    client = boto3.client('ecr')
    response = client.describe_repositories(
        repositoryNames=[repo_name]
    )
    print(response["repositories"][0]["repositoryUri"])


def get_ecr_token():
    client = boto3.client('ecr')
    response = client.get_authorization_token()
    token = response["authorizationData"][0]["authorizationToken"]
    user, password = base64.b64decode(token).decode('utf-8').split(':')
    print(password)


def get_ecr_registry():
    client = boto3.client('ecr')
    response = client.get_authorization_token()
    print(response["authorizationData"][0]["proxyEndpoint"])


def wait_for_tasks_to_start(cluster, service_name, task_revision, number_of_tasks, start_attempts=1):
    max_attempts = int(start_attempts)
    current_attempt = 0
    while True:
        try:
            print("Waiting for all new tasks to reach RUNNING status...")
            wait_for_tasks_to_start_attempt(cluster, service_name, task_revision, number_of_tasks)
            return
        except RuntimeError as err:
            print("Attempt {num} to start all tasks failed".format(num=current_attempt+1))
            print(err)
            current_attempt += 1

            if current_attempt >= max_attempts:
                raise Exception("NEW TASKS DON'T APPEAR TO BE STARTED AFTER {num} ATTEMPTS!".format(num=max_attempts))
            else:
                # ECS has algorithm to exponentially increase time between task start attempts that
                # ultimately # ends up at 15m max.  Need to do similar thing here otherwise calling
                # wait_for_tasks_to_start_attempt will timeout without seeing ECS try to restart a
                # failed task
                wait = current_attempt * 60 if current_attempt < 16 else 16 * 60
                print("Waiting {seconds} before checking tasks again".format(seconds=wait))
                time.sleep(wait)


def wait_for_tasks_to_start_attempt(cluster, service_name, task_revision, number_of_tasks):
    client = boto3.client('ecs')
    new_tasks = []
    max_attempts = 30
    current_attepmt = 0
    while True:
        try:
            task_arns = client.list_tasks(
                cluster=cluster,
                serviceName=service_name,
                desiredStatus='RUNNING'
            )['taskArns']

            tasks = []
            if task_arns:
                tasks = client.describe_tasks(
                    cluster=cluster,
                    tasks=task_arns
                )['tasks']

            new_tasks = [task for task in tasks if task['taskDefinitionArn'].split(':')[-1] == str(task_revision)]
            if len(new_tasks) >= int(number_of_tasks):
                break

            print('.', end='', flush=True)
            time.sleep(2)
            current_attepmt += 1
        except ClientError as err:
            if err.response['Error']['Code'] not in RETRY_EXCEPTIONS:
                raise
            time.sleep(current_attepmt)

        if current_attepmt == max_attempts:
            raise RuntimeError("NEW TASKS DON'T APPEAR TO BE STARTED!")

    max_attempts = 30
    for task in new_tasks:
        current_attepmt = 0
        print("{task} : ".format(task=task['taskArn']), end='', flush=True)
        while True:
            try:
                print('.', end='', flush=True)
                current_task = client.describe_tasks(
                    cluster=cluster,
                    tasks=[task['taskArn']]
                )['tasks'][0]

                if current_task['lastStatus'] == 'RUNNING':
                    break

                if current_task['lastStatus'] == 'STOPPED':
                    raise RuntimeError("{task} STOPPED UNEXPECTEDLY!".format(task=task['taskArn']))

                time.sleep(5)
            except ClientError as err:
                if err.response['Error']['Code'] not in RETRY_EXCEPTIONS:
                    raise
                time.sleep(current_attepmt)

            if current_attepmt == max_attempts:
                raise RuntimeError("{task} DOESN'T APPEAR TO BE RUNNING! ".format(task=task['taskArn']))

        print(" RUNNING!")


def task_status_report(cluster, service_name, service_label):
    client = boto3.client('ecs')
    service = "{service_name}_{service_label}".format(service_name=service_name, service_label=service_label)
    task_arns = client.list_tasks(
        cluster=cluster,
        serviceName=service
    )['taskArns']
    describe_tasks_response = client.describe_tasks(
        cluster=cluster,
        tasks=task_arns
    )
    print("Tasks for service: {}:".format(service))
    print(describe_tasks_response['tasks'])


def update_log_group(env, region, service_name, service_label):
    client = boto3.client('logs')
    log_group_name = "/{env}/psc/{service_name}_{service_label}".format(env=env,
                                                                    service_name=service_name,
                                                                    service_label=service_label)
    log_group_arn = client.describe_log_groups(
        logGroupNamePrefix=log_group_name
    )['logGroups'][0]['arn']

    lambda_client = boto3.client('lambda')

    func_name = '{env}-cloudwatchToKinesis'.format(env=env)

    lambda_arn = lambda_client.get_function(
        FunctionName=func_name
    )['Configuration']['FunctionArn']

    existing_filters = client.describe_subscription_filters(logGroupName=log_group_name)['subscriptionFilters']

    subscribed_to_target = len([filter for filter in existing_filters if filter['destinationArn'] == lambda_arn]) == 1

    subscribed_to_something = len(existing_filters) == 1

    if subscribed_to_something and not subscribed_to_target:
        print("DELETING OLD SUBSCRIPTION")
        client.delete_subscription_filter(
            logGroupName=log_group_name,
            filterName=existing_filters[0]['filterName']
        )

    if not subscribed_to_target:
        print("CREATING SUBSCRIPTION TO {func_name}".format(func_name=func_name))
        client.put_subscription_filter(
            logGroupName=log_group_name,
            filterName=binascii.b2a_hex(os.urandom(10)).decode('utf-8'),
            filterPattern='',
            destinationArn=lambda_arn,
        )


def build_docker_vars(*args):
    print(' '.join(["--build-arg {arg}".format(arg=arg) for arg in args]))


def build_infra_vars(*args):
    tf_var = "TF_VAR_{arg_name}={arg_value}"
    print(' '.join([tf_var.format(arg_name=arg.split('=')[0].lower(), arg_value=arg.split('=')[1]) for arg in args]))


if __name__ == '__main__':
    # Parse arguments from command line
    parser = argparse.ArgumentParser()

    # Set up required arguments this script
    parser.add_argument('function', type=str, help='function to call')
    parser.add_argument('arguments', type=str, nargs='*', help='arguments')

    # Parse given arguments
    args = parser.parse_args()

    # Get the function based on the command line argument and
    # call it with other command line arguments as function arguments
    eval(args.function)(*args.arguments)
