#!/usr/bin/env bash
set -e
DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
source $(pwd)/config/configuration
source /deployment/deploy/helpers/run_command.sh
source /deployment/deploy/helpers/print_line.sh

ti="-i"
test -t 1 && ti="-ti"
add_host=${ADD_HOST}

ARTIFACTORY_URL=${ARTIFACTORY_URL:-artifactory-pub.bit9.local}

AWS_VOLUME="-v $HOME/.aws:/root/.aws"
GIT_VOLUME="-v $(pwd)/.git/:/app_home/.git/"

if [ ! -z "${TF_LOG_LEVEL}" ]; then
    printh "LOG LEVEL ${TF_LOG_LEVEL}"
    TF_LOG_LEVEL="TF_LOG=${TF_LOG_LEVEL} \\"
fi

if [ -z "${TF_LOG_LEVEL}" ]; then
    printh "TF_LOG_LEVEL NOT SET. USING DEFAULT"
fi

if [ ! -z "${AWS_ACCESS_KEY_ID}" ]; then
    AWS_VARS="-e AWS_ACCESS_KEY_ID=$AWS_ACCESS_KEY_ID -e AWS_DEFAULT_REGION=$AWS_DEFAULT_REGION -e AWS_SECRET_ACCESS_KEY=$AWS_SECRET_ACCESS_KEY"
fi
if [ ! -z "${SFX_AUTH_TOKEN}" ]; then
    MISC_VARS="-e SFX_AUTH_TOKEN=${SFX_AUTH_TOKEN}"
fi

if [ ! -z "${PLAN_ONLY}" ]; then
  tf_command="terraform plan"
else
  tf_command="terraform apply -auto-approve"
fi

INFRA_VOLUMES="-v $(pwd)/terraform/:/app_home $AWS_VOLUME $GIT_VOLUME"
INFRA_ENV_COMMON_VOLUMES="-v $(pwd)/terraform/env_common/:/app_home $AWS_VOLUME $GIT_VOLUME"
INFRA_X_ENV_COMMON_VOLUMES="-v $(pwd)/terraform/x_env_common/:/app_home $AWS_VOLUME $GIT_VOLUME"
INFRA_MANUAL_VOLUMES="-v $(pwd)/terraform/manual/${MANUAL_TF_SERVICE}:/app_home $AWS_VOLUME $GIT_VOLUME"

FOLDERS=( "" "/env_common" "/x_env_common" "/manual/${MANUAL_TF_SERVICE}")
STATES=( "state" "remote_state" "remote_env_common_state" "remote_x_env_common_state")

init_tf_infra(){
    INFRA_PATH=$1
    docker run --rm ${ti} $INFRA_PATH $AWS_VARS $MISC_VARS ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "rm -rf /app_home/.terraform"
    if [ ! -z "${AWS_VARS}" ]; then
        AWS_VARS_INIT="-e AWS_ACCESS_KEY_ID=$AWS_ACCESS_KEY_ID -e AWS_DEFAULT_REGION=us-east-1 -e AWS_SECRET_ACCESS_KEY=$AWS_SECRET_ACCESS_KEY"
    fi
    docker run --rm ${ti} $INFRA_PATH $AWS_VARS_INIT $MISC_VARS ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "terraform init"
}

create_env_common_infra(){
    if [ -d $(pwd)/terraform/env_common ] && [ `ls -1 $(pwd)/terraform/env_common/*.tf 2>/dev/null | wc -l` != 0 ]; then
        printp "${CREATE_OR_PLAN} ENV COMMON INFRA"

        docker cp $IMG_ID:/app_home/terraform.d/ $(pwd)/terraform/env_common/terraform.d
        init_tf_infra "$INFRA_ENV_COMMON_VOLUMES"
        docker run --rm ${ti} $INFRA_ENV_COMMON_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "\
            ${TF_LOG_LEVEL}
            TF_VAR_enabled=$SERVICE_ENABLED \
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_desired_count=${INSTANCES} \
            TF_VAR_launch_type=${LAUNCH_TYPE} ${INFRA_VARS}\
            ${tf_command}
        "
    fi
}

create_x_env_common_infra(){
    if [ -d $(pwd)/terraform/x_env_common ] && [ `ls -1 $(pwd)/terraform/x_env_common/*.tf 2>/dev/null | wc -l` != 0 ]; then
        printp "${CREATE_OR_PLAN} X-ENV COMMON INFRA"

        docker cp $IMG_ID:/app_home/terraform.d/ $(pwd)/terraform/x_env_common/terraform.d
        init_tf_infra "$INFRA_X_ENV_COMMON_VOLUMES"
        docker run --rm ${ti} $INFRA_X_ENV_COMMON_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "\
            TF_VAR_enabled=$SERVICE_ENABLED \
            ${TF_LOG_LEVEL}
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_desired_count=${INSTANCES} \
            TF_VAR_launch_type=${LAUNCH_TYPE} ${INFRA_VARS}\
            ${tf_command}
        "
    fi
}

############################################## SERVICE UPDATES ##############################################
create_repo(){
    printp "${CREATE_OR_PLAN} DOCKER REPO"
    docker run --rm ${ti} $INFRA_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder " \
            ${TF_LOG_LEVEL}
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_service_label=${SERVICE_LABEL} \
            TF_VAR_enabled=false ${INFRA_VARS}\
            ${tf_command}
    "
}

build_and_push_image(){
    printp "BUILDING AND PUSHING NEW IMAGE"
    DEPLOY=true . /deployment/build_image_remote.sh
}

remove_task_definition_from_state(){
   printp "REMOVING TASK DEFINITION FOR ${SERVICE_NAME} FROM TERRAFORM STATE"
   cmd="terraform state rm ${1}"
   docker run --rm ${ti} $INFRA_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "${TF_LOG_LEVEL} ${cmd}"            
}

return_task_definition_to_state() {
   printp "IMPORTING TASK DEFINITION FOR ${SERVICE_NAME} TO TERRAFORM STATE"
   cmd="terraform import ${1} ${2}"
   docker run --rm ${ti} $INFRA_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "${TF_LOG_LEVEL} ${cmd}"
}

update_service(){
    set -x
    printp "SERVICE UPDATES FOR: "
    print "ENV: $ENV"
    print "SERVICE_NAME: $SERVICE_NAME"
    print "SERVICE_LABEL: $SERVICE_LABEL"
    print "DESIRED_COUNT: $INSTANCES"
    print "CPU: $CPU"
    print "MEMORY: $MEMORY"

    docker run --rm ${ti} $INFRA_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder " \
            ${TF_LOG_LEVEL}
            TF_VAR_enabled=$SERVICE_ENABLED \
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_port=${PORT} \
            TF_VAR_service_label=${SERVICE_LABEL} \
            TF_VAR_service_weight=${SERVER_WEIGHT} \
            TF_VAR_desired_count=${INSTANCES} \
            TF_VAR_cpu=${CPU} \
            TF_VAR_memory=${MEMORY} \
            TF_VAR_digest=${IMAGE_DIGEST} \
            TF_VAR_launch_type=${LAUNCH_TYPE} ${INFRA_VARS}\
            ${tf_command}
    "
    set +x
}

delete_service(){
    printp "PROVISIONING DELETES"
    docker run --rm ${ti} $INFRA_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder " \
            ${TF_LOG_LEVEL}
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_service_label=${SERVICE_LABEL} \
            TF_VAR_enabled=false ${INFRA_VARS}\
            ${tf_command}
    "
}

delete_env_common(){
    printp "PROVISIONING DELETES"
    docker run --rm ${ti} $INFRA_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder " \
            ${TF_LOG_LEVEL}
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_service_label=blue \
            TF_VAR_enabled=false ${INFRA_VARS}\
            terraform destroy -auto-approve
    "
    docker run --rm ${ti} $INFRA_ENV_COMMON_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder " \
            ${TF_LOG_LEVEL}
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_enabled=false ${INFRA_VARS}\
            terraform init
    "
    docker run --rm ${ti} $INFRA_ENV_COMMON_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder " \
            ${TF_LOG_LEVEL}
            TF_VAR_service_name=${SERVICE_NAME} \
            TF_VAR_enabled=false ${INFRA_VARS}\
            terraform destroy -auto-approve
    "
}


############################################## TERRAFORM SETUP ##############################################
pre_populate_state(){
    IMG_ID=$(docker create ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder)
    if [[ $ENV == *prod* ]] || [[ $ENV == *eap* ]]; then
      BUCKET_NAME=confer-terraform-prod
    else
      BUCKET_NAME=defense-terraform-dev
    fi
    if [[ ${NEW_ENV_MODEL} == true ]]; then
      ENV_BUCKET_NAME=psc-terraform-prod
    else
      ENV_BUCKET_NAME=${ENV_BUCKET_NAME:-$BUCKET_NAME}
    fi
    if [[ $ENV == *hub-prod* ]]; then
      ENV_BUCKET_NAME=psc-terraform-prod
    fi
    X_ENV_BUCKET_NAME=${X_ENV_BUCKET_NAME:-defense-terraform-dev}
    STATE_FILES=
    for folder in "${FOLDERS[@]}"
    do
        for state in "${STATES[@]}"
        do
	    STATE_FILES="$STATE_FILES$(pwd)/terraform$folder/$state "
        done
    done
    ENV_VARS="ENV=$ENV \
	      AWS_DEFAULT_REGION=$AWS_DEFAULT_REGION \
	      AWS_ACCESS_KEY_ID=$AWS_ACCESS_KEY_ID \
	      AWS_SECRET_ACCESS_KEY=$AWS_SECRET_ACCESS_KEY \
              X_REMOTE_ENV=$X_REMOTE_ENV \
              X_ENV_EXT=$X_ENV_EXT \
              X_ENV_BUCKET_NAME=$X_ENV_BUCKET_NAME \
              BUCKET_NAME=$BUCKET_NAME \
              ENV_BUCKET_NAME=$ENV_BUCKET_NAME\
              REGION=us-east-1 \
              SERVICE_NAME=$SERVICE_NAME \
              SERVICE_LABEL=$SERVICE_LABEL"
    run_deploy_command expand_state_files $STATE_FILES
    ENV_VARS=
}

cleanup_state(){
    docker stop $IMG_ID
    docker rm $IMG_ID
    STATE_FILES=
    for folder in "${FOLDERS[@]}"
    do
        for state in "${STATES[@]}"
        do
	    STATE_FILES="$STATE_FILES$(pwd)/terraform$folder/$state.tf "
        done
    done
    run_deploy_command cleanup_state_files $STATE_FILES
}

init_infra(){
    docker pull ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder
    pre_populate_state
    printp "INITIALIZING ${ENV}_${SERVICE_NAME}_${SERVICE_LABEL} TERRAFORM"
    if [ -f $(pwd)/config/infra_vars ]; then
        INFRA_VARS=$(eval echo -e `<$(pwd)/config/infra_vars`)
        INFRA_VARS=$(run_deploy_command build_infra_vars $INFRA_VARS)
    fi
    docker cp $IMG_ID:/app_home/terraform.d/ $(pwd)/terraform/terraform.d
    init_tf_infra "$INFRA_VOLUMES"
}

############################################## WAITING FOR TASKS #######################################
wait_for_tasks_to_run(){
    NEW_REVISION=$1
    DESIRED_SIZE=$2
    SVC_NAME=${SERVICE_NAME}_${SERVICE_LABEL}
    START_ATTEMPTS=${START_ATTEMPTS:-1}
    print "WAITING FOR NEW TASKS TO START UP: "
    print "SERVICE: $SVC_NAME"
    print "NEW_REVISION: $NEW_REVISION"
    print "DESIRED_SIZE: $DESIRED_SIZE"
    run_deploy_command_no_capture wait_for_tasks_to_start $CLUSTER $SVC_NAME $NEW_REVISION $DESIRED_SIZE $START_ATTEMPTS
}

get_infra_output(){
    pre_populate_state > /dev/null
    VOLUMES=""
    if [ $1 == "env_common" ]; then
        VOLUMES=$INFRA_ENV_COMMON_VOLUMES
    fi

    if [ $1 == "x_env_common" ]; then
        VOLUMES=$INFRA_X_ENV_COMMON_VOLUMES
    fi

    if [ ! -z "$VOLUMES" ]; then
        init_tf_infra "$VOLUMES" > /dev/null
        docker run --rm ${ti} $VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "${INFRA_VARS} terraform output $3 $2"
    fi
}


init_manual_infra(){
    if [ -d $(pwd)/terraform/manual/${MANUAL_TF_SERVICE} ]; then
        docker pull ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder
        pre_populate_state
        printp "INITIALIZING MANUAL $MANUAL_TF_SERVICE TERRAFORM"
        if [ -f $(pwd)/config/infra_vars ]; then
            INFRA_VARS=$(eval echo -e `<$(pwd)/config/infra_vars`)
            INFRA_VARS=$(run_deploy_command build_infra_vars $INFRA_VARS)
        fi
        docker cp $IMG_ID:/app_home/terraform.d/ $(pwd)/terraform/manual/${MANUAL_TF_SERVICE}/terraform.d
        init_tf_infra "$INFRA_MANUAL_VOLUMES"
    fi
}

create_manual_infra(){
    if [ -d $(pwd)/terraform/manual/${MANUAL_TF_SERVICE} ]; then
        printp "MANUALLY CREATING ${MANUAL_TF_SERVICE} INFRA"
        docker run --rm ${ti} $INFRA_MANUAL_VOLUMES $AWS_VARS $MISC_VARS  ${ARTIFACTORY_URL}:5000/cbd/gateway-terraform-builder "\
            TF_VAR_service_name=${SERVICE_NAME} ${INFRA_VARS}\
            ${tf_command}
        "
        run_deploy_command cleanup_state_files $STATE_FILES
    fi
}
